--[[
General purpose useful functions.
]]

function printf(text, ...)
	print(string.format(text, ...))
end

function assertf(var, msg, ...)
	assert(var, string.format(msg, ...))
end

function table.copy(t)
	local val = {}

	for k, v in pairs(t) do
		val[k] = v
	end

	return val
end

function math.clamp(num, min, max)
	return math.min(math.max(num, min), max);
end

function math.round(num)
	if num >= 0 then
		return math.floor(num + .5)
	else
		return math.ceil(num - .5)
	end
end

function invoke(func)
	if (func and type(func) == "function") then
		print("Invoking function " .. tostring(func));
		func();
		return true;
	end

	print("Can't invoke the given param.");
	return false;
end

function iff(cond, a, b)
	if (cond) then
		return a;
	else
		return b;
	end
end

function iter(t)
	local i = 0;
	local n = table.getn(t);

	return function()
		i = i + 1;
		if i <= n then
			return t[i];
		end
	end
end

-- Collision detection function.
-- Checks if a and b overlap.
-- w and h mean width and height.
function rectIntersect(ax1, ay1, aw, ah, bx1, by1, bw, bh)
	local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
	return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end