require("code.functions")
require("code.Tard")
local tween = require("libs.tween")
local timer = require("code.Timer")
local res = require("code.ResCache")



-- Shorcuts, coz we r cool.
gf = love.graphics
kb = love.keyboard

-- Globalize delta time so we don't have to pass it to all kind sirs and ladies who need it.
dt = 0

-- Playfield.
local pf = {
	playfield = {},
	W = 7,
	H = 12,
	BLOCK_SIZE = 48,
}

-- Game play.
local gp = {
	level = 0,
	fallDuration = 0,
	fallCycleDone = true,
	FALL_DURATION = 0.25,
	BASE_FALL_PERIOD = 1.25,
	FALL_PERIOD_COEF = 0.15,
	MOVE_DURATION = 0.25,
	fallPeriod = 0,
	compoundX = 0,
	compoundMoving = false,
	EASING_FUNCTION = tween.easing.inOutSine,
	tardsFalling = false,
}

-- Retards.
local listOfTards = {}

-- Compound.
local compound = {}

-- Fonts.
local font



--- Creates a tard.
-- @param id The ID of the tard.
-- @param x The horizontal position on the playing field (not pixel position).
-- @param y The vertical position on the playing field (not pixel position).
function createTard(id, x, y)
	local t = Tard.new(id, x, y)

	table.insert(listOfTards, t)

	return t
end

function renewCompound(tardIDs)
	compound = {}
	local t

	for i = 1, #tardIDs do
		t = createTard(tardIDs[i], 4, i - #tardIDs)
		compound[i] = t
	end

	gp.compoundX = 4
end

function swapCompound()
	local t = table.copy(compound)

	compound[1] = t[#t]
	--compound[1].y = compound[1].y - (#t - 1)
	tween(0.15, compound[1], {y = compound[1].y - (#t - 1)}, gp.EASING_FUNCTION)

	for i = 2, #t do
		compound[i] = t[i - 1]
		--compound[i].y = compound[i].y + 1
		tween(0.15, compound[i], {y = compound[i].y + 1}, gp.EASING_FUNCTION)
	end
end

function moveCompound(dir)
	if not gp.compoundMoving and (dir == -1 and gp.compoundX > 1 or dir == 1 and gp.compoundX < pf.W) then
		for i = 1, #compound do
			tween(gp.MOVE_DURATION, compound[i], {x = compound[i].x + dir}, gp.EASING_FUNCTION)
		end

		--tween(gp.MOVE_DURATION, {val = 0}, {val = 0}, tween.easing.linear, compoundMoveDone, dir)
		timer.new(gp.MOVE_DURATION, 0, compoundMoveDone, dir)
		gp.compoundMoving = true
	end
end

function compoundMoveDone(dir)
	gp.compoundX = gp.compoundX + dir
	gp.compoundMoving = false

	savePlayfield()
end

function releaseCompound()
	renewCompound({1, 2, 1})
end

function setLevel(lvl)
	gp.level = lvl
	--local fd = gp.BASE_FALL_DURATION
	--for i = 1, gp.level do
	--	fd = fd * gp.FALL_DURATION_MOD
	--end

	gp.fallDuration = gp.FALL_DURATION
	gp.fallPeriod = gp.BASE_FALL_PERIOD / (1 + gp.FALL_PERIOD_COEF * gp.level)
end

function savePlayfield()
	-- Clear playfield.
	for x = 1, pf.W do
		for y = 1, pf.H do
			pf.playfield[x][y] = nil
		end
	end

	-- Save playfield state.
	for k, v in pairs(listOfTards) do
		if v.x > 0 and v.x < pf.W and v.y > 0 and v.y < pf.H then
			v.x = math.round(v.x)
			v.y = math.round(v.y)
			--print(v.x, v.y)
			pf.playfield[v.x][v.y] = true
		end
	end
end

function fallTards()
	--if gp.tardsFalling then return end

	savePlayfield()

	-- Fall tards.
	for k, v in pairs(listOfTards) do
		tween(gp.fallDuration, v, {y = v.y + 1}, gp.EASING_FUNCTION)
	end

	gp.tardsFalling = true
	timer.new(gp.fallDuration, 0, fallTardsDone)

	--timer.new(gp.fallDuration * gp.FALL_PERIOD_COEF, 0, fallTards)
	gp.fallCycleDone = false
	timer.new(gp.fallPeriod, 0, fallCycleDone)
end

function fallCycleDone()
	gp.fallCycleDone = true
end

function fallTardsDone()
	gp.tardsFalling = false
end


function love.load()
	local path = ""

	-- Load fonts.
	path = "data/fonts/"
	font = gf.newFont(path .. "tahoma.ttf", 16)

	-- Load images.
	path = "data/pics/"
	res.addImage(path .. "tard1.png", "tard1")
	res.addImage(path .. "tard2.png", "tard2")

	-- Load localization.
	res.loadLoc("data/loc/strings.en.lua")

	-- Initialize the playfield.
	for i = 1, pf.W do
		pf.playfield[i] = {}

		for j = 1, pf.H do
			pf.playfield[i][j] = nil
		end
	end

	--local t = createTard(1, 1, 1)
	--tween(0.5, t, {y = 48}, tween.easing.inOutSine)
	--createTard(2, 5, 2)
	setLevel(1)
	renewCompound({1, 1, 2})
	--fallTards()

	local t = createTard(1, 1, 1)
	tween(3, t, {y = t.y + 1}, gp.EASING_FUNCTION)
	tween(3, t, {y = t.y - 1}, gp.EASING_FUNCTION)
	--timer.new(0.25, 0, function() tween(0.5, t, {y = t.y - 0.5}, gp.EASING_FUNCTION) end)
end

function love.update(deltaTime)
	dt = deltaTime

	tween.update(dt)
	timer.update(dt)

	if gp.fallCycleDone and not gp.tardsFalling and not gp.compoundMoving then
		gp.fallCycleDone = false
		--fallTards()
	end

	-- Keyboard input.
	if kb.isDown("left") then
		moveCompound(-1)
	elseif kb.isDown("right") then
		moveCompound(1)
	end

	--quickFall = kb.isDown("down")
end

function love.draw()
	-- Draw stats.
	gf.setFont(font)
	gf.printf(string.format("Level: %i", gp.level), 4, 4, 1000, "left")
	--gf.print(targetX, 4, 40)
	--gf.print(compound[1].x, 4, 56)

	local bs = pf.BLOCK_SIZE
	local x, y = 0, 0

	-- Draw the playfield.
	gf.push()
	gf.scale(1, 1)
	gf.translate(300, 150)

	for i = 1, pf.W do
		for j = 1, pf.H do
			x = (i - 1) * bs
			y = (j - 1) * bs
			gf.rectangle("line", x, y, bs, bs)
			local txt = iff(pf.playfield[i][j], "yes", "no")
			gf.print(txt, x, y)
		end
	end

	-- Draw tards.
	for k, v in pairs(listOfTards) do
		gf.draw(res.getImage("tard" .. v.id), (v.x - 1) * bs, (v.y - 1) * bs)
	end

	gf.pop()
end

function love.keypressed(key)
	if key == "escape" then
		love.event.quit()
	end

	if key == "up" then
		swapCompound()
	end
end

function love.quit()
	print("fu")
end